@@ -54,7 +54,7 @@ static void dbfile_config_defaults(struct dbfile_config *cfg)
 {
 	memset(cfg, 0, sizeof(*cfg));
 	cfg->blocksize = blocksize;
-	strncpy(cfg->hash_type, hash_type, 8);
+	strncpy(cfg->hash_type, HASH_TYPE, 8);
 }
 
 #if 0
@@ -109,38 +109,32 @@ out:
 	return ret;
 }
 
-int create_indexes_old(sqlite3 *db)
-{
-	int ret;
-
-#define	CREATE_HASHES_DIGEST_INDEX_OLD				\
-"create index if not exists idx_digest on hashes(digest);"
-	ret = sqlite3_exec(db, CREATE_HASHES_DIGEST_INDEX_OLD, NULL, NULL, NULL);
-	if (ret)
-		goto out;
-
-#define	CREATE_HASHES_INOSUB_INDEX_OLD				\
-"create index if not exists idx_hashes_inosub on hashes(ino, subvol);"
-	ret = sqlite3_exec(db, CREATE_HASHES_INOSUB_INDEX_OLD, NULL, NULL, NULL);
-	if (ret)
-		goto out;
-
-#define	CREATE_FILES_INOSUB_INDEX_OLD				\
-"create index if not exists idx_inosub on files(ino, subvol);"
-	ret = sqlite3_exec(db, CREATE_FILES_INOSUB_INDEX_OLD, NULL, NULL, NULL);
-out:
-	return ret;
-}
-
 int create_indexes(sqlite3 *db, struct dbfile_config *cfg)
 {
 	int ret;
 
+	/* The following indexes are version independent */
+#define	CREATE_HASHES_DIGEST_INDEX				\
+"create index if not exists idx_digest on hashes(digest);"
+	ret = sqlite3_exec(db, CREATE_HASHES_DIGEST_INDEX, NULL, NULL, NULL);
+	if (ret)
+		goto out;
+
+#define	CREATE_HASHES_INOSUB_INDEX				\
+"create index if not exists idx_hashes_inosub on hashes(ino, subvol);"
+	ret = sqlite3_exec(db, CREATE_HASHES_INOSUB_INDEX, NULL, NULL, NULL);
+	if (ret)
+		goto out;
+
+	/* Only for hashfile v2 */
 	if (cfg->major == BLOCK_DEDUPE_DBFILE_VER) {
-		ret = create_indexes_old(db);
+#define	CREATE_FILES_INOSUB_INDEX_OLD				\
+"create index if not exists idx_inosub on files(ino, subvol);"
+		ret = sqlite3_exec(db, CREATE_FILES_INOSUB_INDEX_OLD, NULL, NULL, NULL);
 		goto out;
 	}
 
+	/* Only for hashfile v3 */
 #define	CREATE_EXTENTS_DIGEST_INDEX					\
 "create index if not exists idx_extent_digest on extents(digest);"
 	ret = sqlite3_exec(db, CREATE_EXTENTS_DIGEST_INDEX, NULL, NULL, NULL);
@@ -159,12 +153,6 @@ int create_indexes(sqlite3 *db, struct dbfile_config *cfg)
 	if (ret)
 		goto out;
 
-	ret = sqlite3_exec(db, CREATE_HASHES_DIGEST_INDEX_OLD, NULL, NULL, NULL);
-	if (ret)
-		goto out;
-
-	ret = sqlite3_exec(db, CREATE_HASHES_INOSUB_INDEX_OLD, NULL, NULL, NULL);
-
 out:
 	if (ret)
 		perror_sqlite(ret, "creating database index");
@@ -181,6 +169,12 @@ static int dbfile_set_modes(sqlite3 *db)
 		return ret;
 	}
 
+	ret = sqlite3_exec(db, "PRAGMA cache_size = -256000", NULL, NULL, NULL);
+	if (ret) {
+		perror_sqlite(ret, "configuring database (cache size)");
+		return ret;
+	}
+
 	return ret;
 }
 #define MEMDB_FILENAME	"file::memory:?cache=shared"
@@ -838,8 +832,7 @@ static int dbfile_check_version(sqlite3 *db)
 	return 0;
 }
 
-static int __dbfile_store_file_info(sqlite3 *db, sqlite3_stmt *stmt,
-				    struct filerec *file)
+static int __dbfile_store_file_info(sqlite3_stmt *stmt, struct filerec *file)
 {
 	int ret;
 
@@ -898,7 +891,7 @@ int dbfile_store_file_info(sqlite3 *db, struct filerec *file)
 		return ret;
 	}
 
-	ret = __dbfile_store_file_info(db, stmt, file);
+	ret = __dbfile_store_file_info(stmt, file);
 
 	sqlite3_finalize(stmt);
 	return ret;
@@ -925,7 +918,7 @@ int dbfile_sync_files(sqlite3 *db)
 			dprintf("File \"%s\" still needs update in db\n",
 				file->filename);
 
-			ret = __dbfile_store_file_info(db, stmt, file);
+			ret = __dbfile_store_file_info(stmt, file);
 			if (ret)
 				break;
 
@@ -1070,7 +1063,7 @@ int dbfile_store_block_hashes(sqlite3 *db, struct dbfile_config *cfg,
 		if (ret)
 			goto bind_error;
 
-		ret = sqlite3_bind_blob(stmt, 5, digest, digest_len,
+		ret = sqlite3_bind_blob(stmt, 5, digest, DIGEST_LEN,
 					SQLITE_STATIC);
 		if (ret)
 			goto bind_error;
@@ -1156,7 +1149,7 @@ int dbfile_store_extent_hashes(sqlite3 *db, struct dbfile_config *cfg,
 		if (ret)
 			goto bind_error;
 
-		ret = sqlite3_bind_blob(stmt, 7, digest, digest_len,
+		ret = sqlite3_bind_blob(stmt, 7, digest, DIGEST_LEN,
 					SQLITE_STATIC);
 		if (ret)
 			goto bind_error;
@@ -1450,7 +1443,11 @@ int dbfile_load_block_hashes(struct hash_tree *hash_tree)
 
 #define GET_DUPLICATE_HASHES \
 	"SELECT hashes.digest, ino, subvol, loff, flags FROM hashes " \
-	"JOIN (SELECT digest FROM hashes GROUP BY digest " \
+	"JOIN (SELECT DISTINCT digest FROM hashes WHERE digest IN " \
+	"(SELECT DISTINCT digest FROM hashes WHERE ino IN " \
+	"(SELECT DISTINCT ino FROM files WHERE dedupe_seq > " \
+	"(SELECT keyval FROM config WHERE keyname = 'dedupe_sequence')))" \
+	"GROUP BY digest " \
 				"HAVING count(*) > 1) AS duplicate_hashes " \
 	"on hashes.digest = duplicate_hashes.digest;"
 
@@ -1525,7 +1522,11 @@ int dbfile_load_extent_hashes(struct results_tree *res)
 	 */
 #define GET_DUPLICATE_EXTENTS					      \
 	"SELECT extents.digest, ino, subvol, loff, extents.len, poff, flags FROM extents " \
-	"JOIN (SELECT digest,len FROM extents GROUP BY digest,len HAVING count(*) > 1) " \
+	"JOIN (SELECT digest,len FROM extents where digest in " \
+	"(select distinct digest from extents where ino in " \
+	"(select ino from files where dedupe_seq > " \
+	"(select keyval from config where keyname = 'dedupe_sequence'))) " \
+	"GROUP BY digest,len HAVING count(*) > 1) " \
 	"AS duplicate_extents on extents.digest = duplicate_extents.digest AND " \
 	"extents.len = duplicate_extents.len;"
 
@@ -1574,8 +1575,7 @@ out:
 }
 
 int dbfile_load_one_file_extent(sqlite3 *db, struct filerec *file,
-				uint64_t loff, unsigned int len,
-				struct file_extent *extent)
+				uint64_t loff, struct file_extent *extent)
 {
 	int ret;
 	sqlite3_stmt *stmt = NULL;
@@ -1705,7 +1705,8 @@ out:
 	return ret;
 }
 
-static int iter_cb(void *priv, int argc, char **argv, char **column)
+static int iter_cb(void *priv, int argc, char **argv,
+		char **column [[maybe_unused]])
 {
 	iter_files_func func = priv;
 
