@@ -26,7 +26,6 @@
 #include <string.h>
 #include <getopt.h>
 #include <inttypes.h>
-#include <unistd.h>
 
 #include <glib.h>
 
@@ -59,6 +58,8 @@ int v2_hashfile = 0;
 int dedupe_same_file = 1;
 int skip_zeroes = 0;
 
+unsigned int batch_size = 0;
+
 int target_rw = 1;
 static int version_only = 0;
 static int help_option = 0;
@@ -86,8 +87,7 @@ int do_lookup_extents = 1;
 int fiemap_during_dedupe = 1;
 
 int stdout_is_tty = 0;
-
-static char *user_hash = DEFAULT_HASH_STR;
+bool do_block_hash = false;
 
 static void print_file(char *filename, char *ino, char *subvol)
 {
@@ -213,32 +213,13 @@ static void add_exclude_pattern(const char *pattern)
 	}
 }
 
-static void usage(const char *prog)
+static void print_version()
 {
 	char *s = NULL;
 #ifdef	DEBUG_BUILD
 	s = " (debug build)";
 #endif
 	printf("duperemove %s%s\n", VERSTRING, s ? s : "");
-	if (version_only)
-		return;
-
-	printf("Find duplicate extents and optionally dedupe them.\n\n");
-	printf("Basic usage: %s [-r] [-d] [-h] [-v] [-A] "
-	       "[--hashfile=hashfile] OBJECTS\n", prog);
-	printf("\n\"OBJECTS\" is a list of files (or directories) which we\n");
-	printf("want to find duplicate extents in. If a directory is \n");
-	printf("specified, all regular files inside of it will be scanned.\n");
-	printf("\n\t<switches>\n");
-	printf("\t-r\t\tEnable recursive dir traversal.\n");
-	printf("\t-d\t\tDe-dupe the results (must run on a supported fs).\n");
-	printf("\t--hashfile=FILE\tStore hashes in this file.\n");
-	printf("\t-A\t\tOpen files for dedupe in read-only mode.\n");
-	printf("\t-h\t\tPrint numbers in human-readable format.\n");
-	printf("\t-v\t\tPrint extra information (verbose).\n");
-	printf("\t--help\t\tPrints this help text.\n");
-	printf("\n\nPlease see the duperemove(8) manpage for a complete list "
-	       "of options.\n");
 }
 
 static int parse_yesno_option(char *arg, int default_val)
@@ -277,8 +258,6 @@ static int parse_dedupe_opts(const char *opts)
 
 		if (strcmp(token, "same") == 0) {
 			dedupe_same_file = !invert;
-		} else if (strcmp(token, "block") == 0) {
-			; /* This option ignored as of v0.12 */
 		} else if (strcmp(token, "fiemap") == 0) {
 			fiemap_during_dedupe = !invert;
 		} else if (strcmp(token, "partial") == 0) {
@@ -293,7 +272,8 @@ static int parse_dedupe_opts(const char *opts)
 		fprintf(stderr, "Bad dedupe options specified. Valid dedupe "
 			"options are:\n"
 			"\t[no]same\n"
-			"\t[no]block\n");
+			"\t[no]fiemap\n"
+			"\t[no]partial\n");
 		ret = EINVAL;
 	}
 
@@ -312,12 +292,12 @@ enum {
 	IO_THREADS_OPTION,
 	CPU_THREADS_OPTION,
 	LOOKUP_EXTENTS_OPTION,
-	HASH_OPTION,
 	SKIP_ZEROES_OPTION,
 	FDUPES_OPTION,
 	DEDUPE_OPTS_OPTION,
 	QUIET_OPTION,
 	EXCLUDE_OPTION,
+	BATCH_SIZE,
 };
 
 static int add_files_from_stdin(int fdupes)
@@ -397,19 +377,21 @@ static int parse_options(int argc, char **argv, int *filelist_idx)
 		{ "hash-threads", 1, NULL, IO_THREADS_OPTION },
 		{ "cpu-threads", 1, NULL, CPU_THREADS_OPTION },
 		{ "lookup-extents", 1, NULL, LOOKUP_EXTENTS_OPTION },
-		{ "hash", 1, NULL, HASH_OPTION },
 		{ "skip-zeroes", 0, NULL, SKIP_ZEROES_OPTION },
 		{ "fdupes", 0, NULL, FDUPES_OPTION },
 		{ "dedupe-options=", 1, NULL, DEDUPE_OPTS_OPTION },
 		{ "quiet", 0, NULL, QUIET_OPTION },
 		{ "exclude", 1, NULL, EXCLUDE_OPTION },
+		{ "batchsize", 1, NULL, BATCH_SIZE },
 		{ NULL, 0, NULL, 0}
 	};
 
-	if (argc < 2)
-		return 1;
+	if (argc < 2) {
+		help_option = 1;
+		return 0;
+	}
 
-	while ((c = getopt_long(argc, argv, "Ab:vdDrh?LR:q", long_ops, NULL))
+	while ((c = getopt_long(argc, argv, "Ab:vdDrh?LR:qB:", long_ops, NULL))
 	       != -1) {
 		switch (c) {
 		case 'A':
@@ -478,9 +460,6 @@ static int parse_options(int argc, char **argv, int *filelist_idx)
 		case LOOKUP_EXTENTS_OPTION:
 			do_lookup_extents = parse_yesno_option(optarg, 0);
 			break;
-		case HASH_OPTION:
-			user_hash = optarg;
-			break;
 		case SKIP_ZEROES_OPTION:
 			skip_zeroes = 1;
 			break;
@@ -505,8 +484,13 @@ static int parse_options(int argc, char **argv, int *filelist_idx)
 		case EXCLUDE_OPTION:
 			add_exclude_pattern(optarg);
 			break;
+		case BATCH_SIZE:
+		case 'B':
+			batch_size = parse_size(optarg);
+			break;
 		case HELP_OPTION:
 			help_option = 1;
+			break;
 		case '?':
 		default:
 			version_only = 0;
@@ -517,6 +501,9 @@ static int parse_options(int argc, char **argv, int *filelist_idx)
 	if (!do_lookup_extents || !fiemap_during_dedupe)
 		force_v2_hashfile = true;
 
+	if (force_v2_hashfile || partial_extent_search)
+		do_block_hash = true;
+
 	numfiles = argc - optind;
 
 	/* Filter out option combinations that don't make sense. */
@@ -594,7 +581,7 @@ out_nofiles:
 static void print_header(void)
 {
 	vprintf("Using %uK blocks\n", blocksize / 1024);
-	vprintf("Using hash: %s\n", csum_mod->name);
+	vprintf("Using %s hashing\n", v2_hashfile ? "block-based" : "extent-based");
 #ifdef	DEBUG_BUILD
 	printf("Debug build, performance may be impacted.\n");
 #endif
@@ -605,13 +592,13 @@ static int update_config_from_dbfile(void)
 {
 	dedupe_seq = dbfile_cfg.dedupe_seq;
 
-	if (strncasecmp(dbfile_cfg.hash_type, hash_type, 8)) {
+	if (strncasecmp(dbfile_cfg.hash_type, HASH_TYPE, 8)) {
 		fprintf(stderr,
-			"Error: Hashfile %s uses %.*s. for checksums "
-			"but we are using %.*s.\nTry running with "
-			"--hash=%.*s\n", serialize_fname, 8,
-			dbfile_cfg.hash_type, 8, hash_type,
-			8, dbfile_cfg.hash_type);
+			"Error: Hashfile %s uses \"%.*s\" for checksums "
+			"but we are using %.*s.\nYou are probably "
+			"using a hashfile generated from an old version, "
+			"which cannot be read anymore.\n", serialize_fname, 8,
+			dbfile_cfg.hash_type, 8, HASH_TYPE);
 		return EINVAL;
 	}
 
@@ -626,6 +613,74 @@ static int update_config_from_dbfile(void)
 	return 0;
 }
 
+void process_duplicates() {
+	int ret;
+	struct results_tree res;
+	struct hash_tree dups_tree;
+
+	init_results_tree(&res);
+	init_hash_tree(&dups_tree);
+
+	qprintf("Loading only duplicated hashes from hashfile.\n");
+
+	if (v2_hashfile) {
+		ret = dbfile_load_block_hashes(&dups_tree);
+		if (ret)
+			goto out;
+
+		ret = find_all_dupes(&dups_tree, &res);
+		if (ret) {
+			/* Only error for this should be enomem */
+			fprintf(stderr,
+				"Error %d: %s while finding duplicate extents.\n",
+				ret, strerror(ret));
+			goto out;
+		}
+	} else {
+		ret = dbfile_load_extent_hashes(&res);
+		if (ret)
+			goto out;
+
+		printf("Found %llu identical extents.\n", res.num_extents);
+		if (partial_extent_search) {
+			ret = dbfile_load_block_hashes(&dups_tree);
+			if (ret)
+				goto out;
+
+			ret = find_additional_dedupe(&res);
+			if (ret)
+				goto out;
+		}
+	}
+
+	if (run_dedupe) {
+		dedupe_results(&res);
+
+		/*
+		 * Bump dedupe_seq, this effectively marks the files
+		 * in our hashfile as having been through dedupe.
+		 */
+		dedupe_seq++;
+
+		/* Sync to get new dedupe_seq written. */
+		dbfile_cfg.dedupe_seq = dedupe_seq;
+		dbfile_cfg.blocksize = blocksize;
+		dbfile_cfg.onefs_dev = fs_onefs_dev();
+		dbfile_cfg.onefs_fsid = fs_onefs_id();
+		ret = dbfile_sync_config(&dbfile_cfg);
+		if (ret)
+			goto out;
+	} else {
+		print_dupes_table(&res);
+	}
+
+out:
+	free_results_tree(&res);
+	free_hash_tree(&dups_tree);
+}
+
+
+
 static int create_update_hashfile(int argc, char **argv, int filelist_idx)
 {
 	int ret;
@@ -686,16 +741,16 @@ static int create_update_hashfile(int argc, char **argv, int filelist_idx)
 		goto out;
 	}
 
-	ret = populate_tree(&dbfile_cfg);
+	ret = create_indexes(dbfile_get_handle(), &dbfile_cfg);
+	if (ret)
+		goto out;
+
+	ret = populate_tree(&dbfile_cfg, batch_size, &process_duplicates);
 	if (ret) {
 		fprintf(stderr,	"Error while populating extent tree!\n");
 		goto out;
 	}
 
-	ret = create_indexes(dbfile_get_handle(), &dbfile_cfg);
-	if (ret)
-		goto out;
-
 	/*
 	 * File scan from above can cause quite a bit of output, flush
 	 * here in case of logfile.
@@ -713,19 +768,31 @@ out:
 int main(int argc, char **argv)
 {
 	int ret, filelist_idx = 0;
-	struct results_tree res;
-	struct hash_tree dups_tree;
 
 	init_filerec();
-	init_results_tree(&res);
-	init_hash_tree(&dups_tree);
 
 	ret = parse_options(argc, argv, &filelist_idx);
-	if (ret || version_only) {
-		usage(argv[0]);
-		return (version_only || help_option) ? 0 : EINVAL;
+	if (ret) {
+		exit(1);
 	}
 
+	if (version_only) {
+		print_version();
+		exit(0);
+	}
+
+	if (help_option) {
+		execlp("man", "man", "8", "duperemove", NULL);
+	}
+
+	/* Allow larger than unusal amount of open files. On linux
+	 * this should bw increase form 1K to 512K open files
+	 * simultaneously.
+	 *
+	 * On multicore SSD machines it's not hard to get to 1K open
+	 * files.
+	 */
+	increase_limits();
 	/*
 	 * Don't run detection if the user has supplied our cpu counts
 	 * already.
@@ -741,15 +808,6 @@ int main(int argc, char **argv)
 	if (fdupes_mode)
 		return add_files_from_stdin(1);
 
-	ret = init_csum_module(user_hash);
-	if (ret) {
-		if (ret == EINVAL)
-			fprintf(stderr,
-				"Could not initialize hash module \"%s\"\n",
-				user_hash);
-		return ret;
-	}
-
 	if (isatty(STDOUT_FILENO))
 		stdout_is_tty = 1;
 
@@ -761,7 +819,7 @@ int main(int argc, char **argv)
 	switch (use_hashfile) {
 	case H_UPDATE:
 	case H_WRITE:
-		if (serialize_fname)
+		if (serialize_fname && !IS_RELEASE)
 			printf("Warning: The hash file format in Duperemove "
 			       "master branch is under development and may "
 			       "change.\nIf the changes are not backwards "
@@ -802,72 +860,19 @@ int main(int argc, char **argv)
 			goto out;
 
 		print_header();
+
+		process_duplicates();
 		break;
 	default:
 		abort_lineno();
 		break;
 	}
 
-	qprintf("Loading only duplicated hashes from hashfile.\n");
-
-	if (v2_hashfile) {
-		ret = dbfile_load_block_hashes(&dups_tree);
-		if (ret)
-			goto out;
-
-		ret = find_all_dupes(&dups_tree, &res);
-		if (ret) {
-			/* Only error for this should be enomem */
-			fprintf(stderr,
-				"Error %d: %s while finding duplicate extents.\n",
-				ret, strerror(ret));
-			goto out;
-		}
-	} else {
-		ret = dbfile_load_extent_hashes(&res);
-		if (ret)
-			goto out;
-
-		ret = dbfile_load_block_hashes(&dups_tree);
-		if (ret)
-			goto out;
-
-		printf("Found %llu identical extents.\n", res.num_extents);
-		if (partial_extent_search) {
-			ret = find_additional_dedupe(&dups_tree, &res);
-			if (ret)
-				goto out;
-		}
-	}
-
 #ifdef	PRINT_STATS
 	run_filerec_stats();
 #endif
 
-	if (run_dedupe) {
-		dedupe_results(&res, &dups_tree);
-
-		/*
-		 * Bump dedupe_seq, this effectively marks the files
-		 * in our hashfile as having been through dedupe.
-		 */
-		dedupe_seq++;
-
-		/* Sync to get new dedupe_seq written. */
-		dbfile_cfg.dedupe_seq = dedupe_seq;
-		dbfile_cfg.blocksize = blocksize;
-		dbfile_cfg.onefs_dev = fs_onefs_dev();
-		dbfile_cfg.onefs_fsid = fs_onefs_id();
-		ret = dbfile_sync_config(&dbfile_cfg);
-		if (ret)
-			goto out;
-	} else {
-		print_dupes_table(&res);
-	}
-
 out:
-	free_results_tree(&res);
-	free_hash_tree(&dups_tree);
 	free_all_filerecs();
 	dbfile_close();
 
