@@ -20,6 +20,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
+#include <sys/param.h>
 #include <limits.h>
 #include <fcntl.h>
 #include <assert.h>
@@ -65,12 +66,12 @@ static unsigned long long files_to_scan;
 static GMutex io_mutex; /* locks db writes */
 static unsigned int leading_spaces;
 
-#define READ_BUF_LEN (8*1024*1024) // 16m
+#define READ_BUF_LEN (8*1024*1024) // 8MB
 
 struct thread_params {
 	GMutex mutex;
-	int num_files;           /* Total number of files we hashed */
-	int num_hashes;          /* Total number of hashes we hashed */
+	unsigned long long num_files;  /* Total number of files we hashed */
+	unsigned long long num_hashes; /* Total number of hashes we hashed */
 	struct dbfile_config	*dbfile_cfg; /* global dbfile config */
 };
 
@@ -78,6 +79,8 @@ extern int v2_hashfile;
 extern struct dbfile_config dbfile_cfg;
 LIST_HEAD(exclude_list);
 
+extern bool do_block_hash;
+
 static void set_filerec_scan_flags(struct filerec *file)
 {
 	if (!(file->flags & FILEREC_NEEDS_SCAN)) {
@@ -231,11 +234,6 @@ static int __add_file(const char *name, struct stat *st,
 		goto out;
 	}
 
-	if (st->st_size < blocksize) {
-		vprintf("Skipping small file %s\n", name);
-		goto out;
-	}
-
 	ret = access(name, R_OK);
 	if (ret) {
 		fprintf(stderr, "Error %d: %s while accessing file %s. "
@@ -387,7 +385,7 @@ int add_file(const char *name)
 		 * subvolumes. We know how to get a unique fsid though
 		 * so use that in the case where we are on btrfs.
 		 */
-		ret = check_btrfs_get_fsid(abspath, &st, &btrfs_fsid);
+		ret = check_btrfs_get_fsid(abspath, &btrfs_fsid);
 		if (ret) {
 			vprintf("Skipping directory %s due to error %d: %s\n",
 				abspath, ret, strerror(ret));
@@ -554,10 +552,13 @@ static GThreadPool *setup_pool(void *arg, void *function)
 	return pool;
 }
 
-static void run_pool(GThreadPool *pool)
+static void run_pool(GThreadPool *pool, struct filerec *file,
+		unsigned int batch_size)
 {
 	GError *err = NULL;
-	struct filerec *file, *tmp;
+	struct filerec *tmp;
+
+	int counter = 0;
 
 	qprintf("Using %u threads for file hashing phase\n", io_threads);
 
@@ -571,6 +572,14 @@ static void run_pool(GThreadPool *pool)
 				g_error_free(err);
 				err = NULL;
 			}
+
+			/* If batch_size is 0 then batching is disabled */
+			if ( batch_size != 0 ) {
+				counter += 1;
+				if (counter > batch_size) {
+					break;
+				}
+			}
 		}
 	}
 
@@ -629,7 +638,7 @@ static int add_block_hash(struct block_csum **hashes, int *nr_hashes,
 	block_hashes = retp;
 	block_hashes[*nr_hashes].loff = loff;
 	block_hashes[*nr_hashes].flags = flags;
-	memcpy(block_hashes[*nr_hashes].digest, digest, DIGEST_LEN_MAX);
+	memcpy(block_hashes[*nr_hashes].digest, digest, DIGEST_LEN);
 
 	*hashes = retp;
 	(*nr_hashes)++;
@@ -640,11 +649,11 @@ struct csum_ctxt {
 	uint64_t blocks_recorded;
 	char *buf;
 	struct filerec *file;
-	unsigned char digest[DIGEST_LEN_MAX];
+	unsigned char digest[DIGEST_LEN];
 
 	struct block_csum *block_hashes;
 	int nr_block_hashes;
-	unsigned char block_digest[DIGEST_LEN_MAX];
+	unsigned char block_digest[DIGEST_LEN];
 };
 
 static int csum_blocks(struct csum_ctxt *data, struct running_checksum *csum,
@@ -663,17 +672,19 @@ static int csum_blocks(struct csum_ctxt *data, struct running_checksum *csum,
 		if (!(skip_zeroes && is_block_zeroed(buf, cmp_len))) {
 			checksum_block(buf, cmp_len, data->block_digest);
 
-			ret = add_block_hash(&data->block_hashes,
-					     &data->nr_block_hashes,
-					     extoff + start,
-					     data->block_digest,
-					     flags);
-			if (ret)
-				break;
+			if (do_block_hash) {
+				ret = add_block_hash(&data->block_hashes,
+						     &data->nr_block_hashes,
+						     extoff + start,
+						     data->block_digest,
+						     flags);
+				if (ret)
+					break;
+			}
 
 			if (!v2_hashfile &&
 			    dbfile_cfg.extent_hash_src == EXTENT_HASH_SRC_DIGEST)
-				add_to_running_checksum(csum, digest_len,
+				add_to_running_checksum(csum, DIGEST_LEN,
 							data->block_digest);
 		}
 
@@ -742,7 +753,7 @@ static int csum_extent(struct csum_ctxt *data, uint64_t extent_off,
 	return ret;
 }
 
-static void csum_whole_file_init(void *location, struct filerec *file,
+static void csum_whole_file_init(struct filerec *file,
 				 struct fiemap_ctxt **fc)
 {
 	static long long unsigned _cur_scan_files;
@@ -782,10 +793,7 @@ static int fiemap_helper(struct fiemap_ctxt *fc, struct filerec *file,
 
 	if ((skip_zeroes && *flags & FIEMAP_EXTENT_UNWRITTEN) ||
 	    (*flags & FIEMAP_SKIP_FLAGS)) {
-		/*
-		 * Unritten or other extent we don't
-		 * want to read
-		 */
+		/* Unritten or other extent we don't want to read */
 		return 1;
 	}
 	return 0;
@@ -796,9 +804,11 @@ static int csum_by_block(struct csum_ctxt *ctxt, struct fiemap_ctxt *fc,
 {
 	int ret;
 	uint64_t loff, poff, fieloff, bytes_read;
-	unsigned int fieflags, fielen;
+	unsigned int fieflags, fielen, read_size = blocksize;
 	struct filerec *file = ctxt->file;
 	struct block_csum *block_hashes;
+	uint64_t size = file->size;
+
 
 	block_hashes = malloc(sizeof(struct block_csum));
 	if (block_hashes == NULL)
@@ -807,22 +817,30 @@ static int csum_by_block(struct csum_ctxt *ctxt, struct fiemap_ctxt *fc,
 	ctxt->block_hashes = block_hashes;
         loff = fieloff = fielen = 0;
 	fieflags = 0;
-	while (loff < file->size) {
+	while (loff < size) {
 		if (fc && loff >= (fieloff + fielen)) {
 			ret = fiemap_helper(fc, file, &poff, &fieloff, &fielen,
 					    &fieflags);
 			if (ret < 0)
 				return ret;
+			/*
+			 * Cap loop to the size of the last _real_ extent.
+			 * Applies to truncated files
+			 */
+			if (fieflags & FIEMAP_EXTENT_LAST)
+				size = fieloff + fielen;
+
 			if (ret == 1) {
 				loff = fieloff + fielen;
 				continue;
 			}
 			loff = fieloff;
-			continue;
+
 		}
 
 //		printf("loff %"PRIu64"\n", loff);
-		ret = csum_extent(ctxt, loff, blocksize, fieflags, &bytes_read);
+		read_size = MAX(fielen, blocksize);
+		ret = csum_extent(ctxt, loff, read_size, fieflags, &bytes_read);
 		if (ret == 0) /* EOF */
 			break;
 
@@ -838,7 +856,7 @@ static int csum_by_block(struct csum_ctxt *ctxt, struct fiemap_ctxt *fc,
 			 */
 			return -1;
 		}
-		loff += blocksize;
+		loff += bytes_read;
 		if (bytes_read < blocksize) {
 			/* Partial read, don't get any more blocks */
 			break;
@@ -948,7 +966,7 @@ static int csum_by_extent(struct csum_ctxt *ctxt, struct fiemap_ctxt *fc,
 		extent_hashes[nb_hash].len = bytes_read;
 		extent_hashes[nb_hash].flags = flags;
 		memcpy(extent_hashes[nb_hash].digest, ctxt->digest,
-		       DIGEST_LEN_MAX);
+		       DIGEST_LEN);
 		nb_hash++;
 
 		ctxt->blocks_recorded += (bytes_read + (blocksize - 1))/blocksize;
@@ -985,7 +1003,7 @@ static void csum_whole_file(struct filerec *file,
 	assert(csum_ctxt.buf != NULL);
 	csum_ctxt.file = file;
 
-	csum_whole_file_init(params, file, &fc);
+	csum_whole_file_init(file, &fc);
 
 	db = dbfile_get_handle();
 	if (!db)
@@ -1029,12 +1047,14 @@ static void csum_whole_file(struct filerec *file,
 			goto err;
 		}
 	} else {
-		ret = dbfile_store_block_hashes(db, params->dbfile_cfg, file,
-						csum_ctxt.nr_block_hashes,
-						csum_ctxt.block_hashes);
-		if (ret) {
-			g_mutex_unlock(&io_mutex);
-			goto err;
+		if (do_block_hash) {
+			ret = dbfile_store_block_hashes(db, params->dbfile_cfg, file,
+							csum_ctxt.nr_block_hashes,
+							csum_ctxt.block_hashes);
+			if (ret) {
+				g_mutex_unlock(&io_mutex);
+				goto err;
+			}
 		}
 
 		ret = dbfile_store_extent_hashes(db, params->dbfile_cfg, file,
@@ -1104,12 +1124,15 @@ err_noclose:
 	return;
 }
 
-int populate_tree(struct dbfile_config *cfg)
+int populate_tree(struct dbfile_config *cfg, unsigned int batch_size,
+		void (*callback)(void))
 {
 	GThreadPool *pool;
 	struct thread_params params;
 	int ret = 0;
 
+	struct filerec *file = NULL;
+
 	g_mutex_init(&params.mutex);
 	params.dbfile_cfg = cfg;
 	params.num_files = params.num_hashes = 0;
@@ -1117,16 +1140,23 @@ int populate_tree(struct dbfile_config *cfg)
 	leading_spaces = num_digits(files_to_scan);
 
 	if (files_to_scan) {
-		pool = setup_pool(&params, csum_whole_file);
-		if (!pool) {
-			ret = ENOMEM;
-			goto out;
+		while (params.num_files != files_to_scan) {
+			pool = setup_pool(&params, csum_whole_file);
+			if (!pool) {
+				ret = ENOMEM;
+				goto out;
+			}
+
+			run_pool(pool, file, batch_size);
+			callback();
 		}
 
-		run_pool(pool);
-
-		printf("Total files:  %d\n", params.num_files);
-		qprintf("Total extent hashes: %d\n", params.num_hashes);
+		printf("Total files scanned:  %llu\n", params.num_files);
+		qprintf("Total extent hashes scanned: %llu\n", params.num_hashes);
+	} else {
+		// Maybe some files where scanned in a previous run but never
+		// deduped
+		callback();
 	}
 out:
 	g_mutex_clear(&params.mutex);
