@@ -34,13 +34,14 @@ dev_t fs_onefs_dev(void);
 uint64_t fs_onefs_id(void);
 
 struct dbfile_config;
-int populate_tree(struct dbfile_config *cfg);
+int populate_tree(struct dbfile_config *cfg, unsigned int batch_size,
+		void (*callback)(void));
 
 /* For dbfile.c */
 struct block_csum {
 	uint64_t	loff;
 	unsigned int	flags;
-	unsigned char	digest[DIGEST_LEN_MAX];
+	unsigned char	digest[DIGEST_LEN];
 };
 
 struct extent_csum {
@@ -48,7 +49,7 @@ struct extent_csum {
 	uint64_t	poff;
 	uint32_t	len;
 	unsigned int	flags;
-	unsigned char	digest[DIGEST_LEN_MAX];
+	unsigned char	digest[DIGEST_LEN];
 };
 
 struct exclude_file {
