@@ -574,12 +574,17 @@ static int push_extents(struct results_tree *res)
 	return 0;
 }
 
-void dedupe_results(struct results_tree *res, struct hash_tree *hashes)
+void dedupe_results(struct results_tree *res)
 {
 	int ret;
 	struct dedupe_counts counts = { 0ULL, };
 	GError *err = NULL;
 
+	/*
+	 * dedupe_results() could be called multiple times, so we reset that bit
+	 * to its initial value every time
+	 */
+	curr_dedupe_pass = 0;
 	results_tree = res;
 
 	print_dupes_table(res);
